@@ -1,5 +1,5 @@
 /*  lmpc -- the Little Movie Processing Centre
-    Copyright (C) 1994-98 Uwe Girlich
+    Copyright (C) 1994-99 Uwe Girlich
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
     Uwe Girlich
-    Erika-von-Brockdorff-Strasse 2
+    Dantestrasse 20
     04159 Leipzig
     Deutschland / Germany
     E-mail: Uwe.Girlich@itp.uni-leipzig.de */
@@ -208,7 +208,7 @@ float ReadAngle(BB_t* m)
 
 void WriteAngle(BB_t* m, float a)
 {
-  put_signed_char(m, a * 256.0 / 360.0);
+  put_signed_char(m, (signed char)(a * 256.0 / 360.0));
 }
 
 float ReadCoord(BB_t* m)
@@ -218,22 +218,27 @@ float ReadCoord(BB_t* m)
 
 void WriteCoord(BB_t* m, float a)
 {
-  put_short(m, a * 8.0);
+  put_short(m, (short)(a * 8.0));
 }   
 
 void WriteVel(BB_t* m, float a)
 {
-  put_signed_char(m, a * 16.0);
+  put_signed_char(m, (signed char)(a * 16.0));
 }
 
 void WriteVol(BB_t* m, float a)
 {
-  put_unsigned_char(m, a * 255.0 + 0.5);
+  put_unsigned_char(m, (unsigned char)(a * 255.0 + 0.5));
 }
 
 void WriteAttenuation(BB_t* m, float a)
 {
-  put_unsigned_char(m, a * 64.0);
+  put_unsigned_char(m, (unsigned char)(a * 64.0));
+}
+
+void WriteTimeofs(BB_t* m, float a)
+{
+  put_unsigned_char(m, (unsigned char)(a * 1000.0));
 }
 
 char* ReadString(BB_t* m, char* buffer)
@@ -253,7 +258,7 @@ float ReadAngle16(BB_t* m)
   
 void WriteAngle16(BB_t* m, float a)
 {
-  put_short(m, a * 65536.0 / 360.0);
+  put_short(m, (short)(a * 65536.0 / 360.0));
 }
 
 float* ReadPosition(BB_t* m, float* a)
@@ -496,7 +501,7 @@ float ReadBlend(BB_t* m)
 
 void WriteBlend(BB_t* m, float a)
 {
-  WriteByte(m, a * 255.0 );
+  WriteByte(m, (unsigned char)(a * 255.0));
 }
 
 float* ReadBlendVec(BB_t* m, float* a)
@@ -521,7 +526,7 @@ float ReadOffset(BB_t* m)
 
 void WriteOffset(BB_t* m, float a)
 {
-  WriteChar(m, a * 4.0 );
+  WriteChar(m, (char)(a * 4.0) );
 }
 
 float* ReadOffsetVec(BB_t* m, float* a)
@@ -665,6 +670,9 @@ void do_simple_argument_write_bin(node* a, BB_t* m)
         case H_ATTENUATION:
           WriteAttenuation(m,*(float*)a->down);
         break;
+        case H_TIMEOFS:
+          WriteTimeofs(m,*(float*)a->down);
+        break;
         case H_BYTEFLOAT:
           WriteChar(m,(char)*(float*)a->down);
         break;
