@@ -1,5 +1,5 @@
 /*  lmpc -- the Little Movie Processing Centre
-    Copyright (C) 1994-98 Uwe Girlich
+    Copyright (C) 1994-99 Uwe Girlich
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
                                                 
     Uwe Girlich
-    Erika-von-Brockdorff-Strasse 2
+    Dantestrasse 20
     04159 Leipzig
     Deutschland / Germany
     E-mail: Uwe.Girlich@itp.uni-leipzig */
@@ -145,7 +145,7 @@ int SendUDPPacket(BB_t* m, unsigned short type,
                   int fromsd, struct sockaddr_in *toaddr, int tolen);
 int SendAckPacket(unsigned long sequence, int sd, struct sockaddr_in *toaddr, int tolen);
 int ReceiveUDPPacket(BB_t* m, unsigned short *type,
-                     int sd, struct sockaddr_in *fromaddr, int *fromlen);
+                     int sd, struct sockaddr_in *fromaddr, socklen_t *fromlen);
 int EqualAddrIn(struct sockaddr_in* a, struct sockaddr_in* b);
 void ControlServer(void);
 pid_t SpawnServer(char* demfilename, unsigned short int port);
@@ -163,7 +163,8 @@ int CreateUDPSocket(unsigned short int  * port)
 {
   struct sockaddr_in me ;
   struct sockaddr_in sin;
-  int sd, len, zz;
+  int sd, zz;
+  socklen_t len;
   
   me = sin = INET_ZERO;
   me.sin_port = htons((unsigned short) *port);
@@ -219,7 +220,7 @@ int SendAckPacket(unsigned long sequence, int sd, struct sockaddr_in *toaddr,
 }
 
 int ReceiveUDPPacket(BB_t* m, unsigned short *type,
-                     int sd, struct sockaddr_in *fromaddr, int *fromlen)
+                     int sd, struct sockaddr_in *fromaddr, socklen_t *fromlen)
 {        
   int receivebytes;
                
@@ -262,7 +263,7 @@ void ControlServer(void)
   unsigned short packet_type;
   long opcode;
   struct sockaddr_in from;
-  int fromlen;
+  socklen_t fromlen;
   
   qsdb.sd = CreateUDPSocket(&(qsdb.port));
   
@@ -407,7 +408,7 @@ void ControlServer(void)
             WriteString(&r, qsdb.shm[player_number].name);
             WriteLong(&r, qsdb.shm[player_number].colors); 
             WriteLong(&r, qsdb.shm[player_number].frags); 
-            WriteLong(&r, realtime() - qsdb.shm[player_number].logintime); 
+            WriteLong(&r, (long)(realtime() - qsdb.shm[player_number].logintime));
             WriteString(&r, qsdb.shm[player_number].address); 
             r.size = r.p - r.buffer;
 
@@ -487,7 +488,7 @@ pid_t PlayServer(int player)
   BB_t am;
   unsigned short atype;
   struct sockaddr_in aaddr;
-  int alen;
+  socklen_t alen;
   unsigned long aseq;
   int runtime;
   shm_t *pls;
@@ -630,7 +631,7 @@ pid_t PlayServer(int player)
 
     ofs_time = 0.0; /* we don't know the offset yet */
     cur_dem_time = 0.0; /* the DEM time starts at 0 */
-    while (ftell(d.file) < d.filesize) {
+    while (ftell(d.file) < (long)d.filesize) {
       double wait_time; 
       fd_set fds;
       struct timeval tv;
@@ -667,8 +668,8 @@ pid_t PlayServer(int player)
         if (wait_time>0.0) {
           FD_ZERO(&fds);
           FD_SET(sd, &fds); 
-          tv.tv_sec = floor(wait_time);
-          tv.tv_usec = 1e6 * (wait_time-tv.tv_sec);
+          tv.tv_sec = (int) floor(wait_time);
+          tv.tv_usec = (int) (1e6 * (wait_time-tv.tv_sec));
           if (select(sd+1, &fds, NULL, NULL, &tv) < 0) syserror(errno,"select");
           if (FD_ISSET(sd, &fds)) {
             ReceiveUDPPacket(&am, &atype, sd, &aaddr, &alen);
@@ -747,7 +748,7 @@ int SendInitDEMPacket(BB_t *pd, unsigned long *sequence,
         BB_t am;
         unsigned short atype;
         struct sockaddr_in aaddr;
-        int alen;
+        socklen_t alen;
         unsigned long aseq;
         int data;
  
@@ -858,7 +859,7 @@ int find_free(void)
 void syntax(void)
 {
   fprintf(stderr,"Demo Broadcasting Server\n");
-  fprintf(stderr,"DBS (c) U. Girlich, 1997-98, Release %s %s (%s)\n", VERSION, DATE, COMMENT);
+  fprintf(stderr,"DBS (c) U. Girlich, 1997-99, Release %s %s (%s)\n", VERSION, DATE, COMMENT);
   fprintf(stderr,"dbs [option ...] demfile\n");
   fprintf(stderr,"-h, --help                   display this help and exit.\n");
   fprintf(stderr,"-v, --version                output version information and exit.\n");
