@@ -1,5 +1,5 @@
 /*  lmpc -- the Little Movie Processing Centre
-    Copyright (C) 1994-98 Uwe Girlich
+    Copyright (C) 1994-99 Uwe Girlich
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
     Uwe Girlich
-    Erika-von-Brockdorff-Strasse 2
+    Dantestrasse 20
     04159 Leipzig
     Deutschland / Germany
     E-mail: Uwe.Girlich@itp.uni-leipzig.de */
@@ -81,24 +81,6 @@ token_t DEM_token[]={
 	{ "cdtrack",            TOKEN_CDTRACK, c_cdtrack },
 	{ "sellscreen",         TOKEN_SELLSCREEN, c_sellscreen },
 	{ "cutscene",		TOKEN_CUTSCENE, c_cutscene },
-/* Hexen II only part */
-	{ "particle2",		TOKEN_PARTICLE2, c_particle2 },
-	{ "midi_name",		TOKEN_MIDI_NAME, c_midi_name },
-	{ "updateclass",	TOKEN_UPDATECLASS, c_updateclass },
-	{ "particle3",		TOKEN_PARTICLE3, c_particle3 },
-	{ "particle4",		TOKEN_PARTICLE4, c_particle4 },
-	{ "set_view_flags",	TOKEN_SET_VIEW_FLAGS, c_set_view_flags },
-	{ "clear_view_flags",	TOKEN_CLEAR_VIEW_FLAGS, c_clear_view_flags },
-	{ "start_effect",	TOKEN_START_EFFECT, c_start_effect },
-	{ "end_effect",		TOKEN_END_EFFECT, c_end_effect },
-	{ "plaque",		TOKEN_PLAQUE, c_plaque },
-	{ "particle_explosion",	TOKEN_PARTICLE_EXPLOSION, c_particle_explosion },
-	{ "set_view_tint",	TOKEN_SET_VIEW_TINT, c_set_view_tint },
-	{ "reference",		TOKEN_REFERENCE, c_reference },
-	{ "clear_edicts",	TOKEN_CLEAR_EDICTS, c_clear_edicts },
-	{ "update_inv",		TOKEN_UPDATE_INV, c_update_inv },
-/* end of the Hexen II only part */
-	
 	{ "updateentity",       TOKEN_UPDATEENTITY, 0 },
 
 	{ "id",                 TOKEN_ID, 0 },
@@ -180,7 +162,6 @@ token_t DEM_token[]={
 	{ "origin_y",           TOKEN_ORIGIN_Y, 0 },
 	{ "origin_z",           TOKEN_ORIGIN_Z, 0 },
 	{ "new",                TOKEN_NEW, 0 },
-	{ "class",		TOKEN_CLASS, 0 },
 		
         { NULL, 0, 0}
 };
@@ -212,14 +193,20 @@ void DEM_init(DEM_t* d, char *filename, char *mode)
   d->lastArmor = 0;
   d->lastMinusHealth = 0;
   d->lastMinusArmor = 0;
-  d->game = QUAKE; /* we will find out a Hexen II recording later */
+  d->game = QUAKE;
 
   /* we need a rewind here because some fopen modes may seek at the end */
   rewind(d->file);
 
-  if (analyzeDEM(d->filename, &(d->datastart), &(d->tics)) < 0)
-    syserror(errno, d->filename);
-  d->datasize = d->filesize - d->datastart;
+  /* don't check output files or pipes */
+  if (strchr(mode,'w') || buf.st_mode & S_IFIFO) {
+    d->datasize = 0;
+  }
+  else {
+    if (analyzeDEM(d->filename, &(d->datastart), &(d->tics)) < 0)
+      syserror(errno, d->filename);
+    d->datasize = d->filesize - d->datastart;
+  } 
 }
 
 void DEM_prepare_write_bin(DEM_t* d)
@@ -271,7 +258,7 @@ void DEM_block_read_bin(DEM_t* d, BB_t* m)
   m->end = &(m->buffer[16]);
   m->size = ReadLong (m);
   if (m->size>DEM_MAX_MESSAGE_SIZE) syserror(WDEM,d->filename);
-  if (fread(m->buffer+16,1,m->size,d->file)!=m->size) syserror(FIREAD,d->filename);
+  if (fread(m->buffer+16,1,m->size,d->file)!=(size_t)m->size) syserror(FIREAD,d->filename);
   m->size += 16; /* total size */
   m->p = m->buffer;
   m->end = &(m->buffer[m->size]);
@@ -289,7 +276,7 @@ node* DEM_bin_to_node(BB_t* m, int opt)
   ca1   = ReadFloat(m);
   ca2   = ReadFloat(m);
   ca3   = ReadFloat(m);
-  n=node_triple_command_init(TOKEN_CAMERA,V_FLOAT,H_ANGLE,
+  n=node_triple_command_init(TOKEN_CAMERA,V_FLOAT,H_FLOAT,
                              NODE_VALUE_FLOAT_dup(ca1),
                              NODE_VALUE_FLOAT_dup(ca2),
                              NODE_VALUE_FLOAT_dup(ca3),
@@ -386,42 +373,8 @@ node* do_message_read_bin(BB_t* m)
     case c_cdtrack:          n=do_cdtrack_message_read_bin(m);           break;
     /* 0x21 */
     case c_sellscreen:       n=do_sellscreen_message_read_bin() ;        break;
-    /* 0x23 */
-    case c_cutscene:         n=do_cutscene_message_read_bin(m);          break;
-    /* Hexen II only part */
     /* 0x22 */
-    case c_particle2:		n=do_particle2_message_read_bin(m);		break;
-    /* 0x23 */
-    /* case c_cutscene:         n=do_cutscene_message_read_bin(m);          break; */
-    /* 0x24 */
-    case c_midi_name:		n=do_midi_name_message_read_bin(m);		break;
-    /* 0x25 */
-    case c_updateclass:		n=do_updateclass_message_read_bin(m);		break;
-    /* 0x26 */
-    case c_particle3:		n=do_particle3_message_read_bin(m);		break;
-    /* 0x27 */
-    case c_particle4:		n=do_particle4_message_read_bin(m);		break;
-    /* 0x28 */
-    case c_set_view_flags:	n=do_set_view_flags_message_read_bin(m);	break;
-    /* 0x29 */
-    case c_clear_view_flags:	n=do_clear_view_flags_message_read_bin(m);	break;
-    /* 0x2A */
-    case c_start_effect:	n=do_start_effect_message_read_bin(m);		break;
-    /* 0x2B */
-    case c_end_effect:		n=do_end_effect_message_read_bin(m);		break;
-    /* 0x2C */
-    case c_plaque:		n=do_plaque_message_read_bin(m);		break;
-    /* 0x2D */
-    case c_particle_explosion:	n=do_particle_explosion_message_read_bin(m);	break;
-    /* 0x2E */
-    case c_set_view_tint:	n=do_set_view_tint_message_read_bin(m);		break;
-    /* 0x2F */
-    case c_reference:		n=do_reference_message_read_bin(m);		break;
-    /* 0x30 */
-    case c_clear_edicts:	n=do_clear_edicts_message_read_bin(m);		break;
-    /* 0x31 */
-    case c_update_inv:		n=do_update_inv_message_read_bin(m);		break;
-    /* end of Hexen II only part */
+    case c_cutscene:         n=do_cutscene_message_read_bin(m);          break;
     default: if (id>=0x80) 
                n=do_updateentity_message_read_bin(m,id & 0x7F);
              else 
@@ -545,8 +498,8 @@ int isDEMtxt(char* filename)
   */
   #define BUFFER_SIZE 1000
   char buffer[BUFFER_SIZE];
-  unsigned long blocksize;
-  int p;
+  long blocksize;
+  long p;
   int state;
 
   if ((file=fopen(filename, "rb"))==NULL) return 0;
@@ -742,7 +695,7 @@ node* do_time_message_read_bin(BB_t* m)
 node* do_print_message_read_bin(BB_t* m)
 { 
   /* variables */
-  char text[1000];
+  char text[MAX_STRING_SIZE];
 
   /* binary in */
   ReadString(m, text);
@@ -756,7 +709,7 @@ node* do_print_message_read_bin(BB_t* m)
 node* do_stufftext_message_read_bin(BB_t* m)
 {
   /* variables */
-  char text[1000];
+  char text[MAX_STRING_SIZE];
 
   /* binary in */
   ReadString(m, text);
@@ -794,13 +747,13 @@ node* do_serverinfo_message_read_bin(BB_t* m)
   node *n, *tn;
 
   long i;
-  char buf[1000];
+  char buf[MAX_STRING_SIZE];
 
   /* variables */
   long serverversion;
   long maxclients;
   long multi;
-  char mapname[1000];
+  char mapname[MAX_STRING_SIZE];
 
   /* binary in */
   serverversion = ReadLong(m);
@@ -820,7 +773,7 @@ node* do_serverinfo_message_read_bin(BB_t* m)
     ReadString(m, buf);
     if (strlen(buf)==0) break;
     DEMTOP->nummodels++;
-    DEMTOP->precache_models = realloc(DEMTOP->precache_models,
+    DEMTOP->precache_models = (char**)realloc(DEMTOP->precache_models,
                                       (1+DEMTOP->nummodels)*sizeof(char*));
     if (DEMTOP->precache_models==NULL) 
       syserror(ENOMEM,"realloc precache_models");
@@ -828,7 +781,6 @@ node* do_serverinfo_message_read_bin(BB_t* m)
     if (DEMTOP->precache_models[DEMTOP->nummodels]==NULL) syserror(errno, "strdup");
   } while (*buf);
   /* fprintf(stderr,"nummodels=%ld\n",DEMTOP->nummodels); */
-  if (DEMTOP->game==QUAKE && DEMTOP->nummodels>255) DEMTOP->game=HEXENII;
   /* free the old strings (multiple level recordings) */
   if (DEMTOP->numsounds) {
     for (i=1;i<=DEMTOP->numsounds;i++) {
@@ -842,7 +794,7 @@ node* do_serverinfo_message_read_bin(BB_t* m)
     ReadString(m, buf);
     if (strlen(buf)==0) break;
     DEMTOP->numsounds++;
-    DEMTOP->precache_sounds = realloc(DEMTOP->precache_sounds,
+    DEMTOP->precache_sounds = (char**)realloc(DEMTOP->precache_sounds,
                                       (1+DEMTOP->numsounds)*sizeof(char*));
     if (DEMTOP->precache_sounds==NULL)
       syserror(ENOMEM,"realloc precache_sounds");
@@ -880,7 +832,7 @@ node* do_lightstyle_message_read_bin(BB_t* m)
 
   /* variables */
   long style;
-  char string[1000]; 
+  char string[MAX_STRING_SIZE]; 
 
   /* binary in */
   style = ReadByte(m);
@@ -901,7 +853,7 @@ node* do_updatename_message_read_bin(BB_t* m)
 
   /* variables */
   long player;
-  char netname[1000];
+  char netname[MAX_STRING_SIZE];
 
   /* binary in */
   player = ReadByte(m);
@@ -1314,75 +1266,36 @@ node* do_spawnstatic_message_read_bin(BB_t* m)
   vec3_t default_angles;
 
   int i;
-  long data1, data2, data3;
 
   /* binary in */
-  if (DEMTOP->game==HEXENII) {
-    default_modelindex = ReadByte(m);
-    default_modelindex |= (ReadByte(m) << 8);
-    default_frame = ReadByte(m);
-    default_colormap = ReadByte(m);
-    default_skin = ReadByte(m);
-    data1=ReadByte(m);
-    data2=ReadByte(m);
-    data3=ReadByte(m);
-    for (i=0 ; i<3 ; i++) {
-      default_origin[i] = ReadCoord(m);
-      default_angles[i] = ReadAngle(m);
-    }
-    /* construct node tree and return the root of it */
-    n = node_command_init(TOKEN_DEFAULT_MODELINDEX,V_INT,H_SHORT,NODE_VALUE_INT_dup(default_modelindex),0);
-    if (default_modelindex >= 1 &&
-        default_modelindex <= DEMTOP->nummodels &&
-        DEMTOP->precache_models[default_modelindex][0]!='*') {
-      node_add_comment(n,NODE_VALUE_STRING_dup(DEMTOP->precache_models[default_modelindex]));
-    }
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_FRAME,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_frame),0));
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_COLORMAP,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_colormap),0));
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_SKIN,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_skin),0));
-    node_add_next(n,node_triple_command_init(TOKEN_DATA,V_BYTEHEX,H_BYTE,
-                    NODE_VALUE_INT_dup(data1),NODE_VALUE_INT_dup(data2),NODE_VALUE_INT_dup(data3),0));
-    node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ORIGIN, V_FLOAT, H_COORD,
-                    NODE_VALUE_FLOAT_dup(default_origin[0]),
-                    NODE_VALUE_FLOAT_dup(default_origin[1]),
-                    NODE_VALUE_FLOAT_dup(default_origin[2]), 0));
-    node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ANGLES, V_FLOAT, H_ANGLE,
-                    NODE_VALUE_FLOAT_dup(default_angles[0]),
-                    NODE_VALUE_FLOAT_dup(default_angles[1]),
-                    NODE_VALUE_FLOAT_dup(default_angles[2]), 0));
-    return node_init_all(TOKEN_SPAWNSTATIC, H_DEM_SPAWNSTATIC, n, 0);
-  }  
-  else {
-    /* binary in */
-    default_modelindex = ReadByte(m);
-    default_frame = ReadByte(m);
-    default_colormap = ReadByte(m);
-    default_skin = ReadByte(m);
-    for (i=0 ; i<3 ; i++) {
-      default_origin[i] = ReadCoord(m);
-      default_angles[i] = ReadAngle(m);
-    }
-
-    /* construct node tree and return the root of it */
-    n = node_command_init(TOKEN_DEFAULT_MODELINDEX,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_modelindex),0);
-    if (default_modelindex >= 1 &&
-        default_modelindex <= DEMTOP->nummodels &&
-        DEMTOP->precache_models[default_modelindex][0]!='*') {
-      node_add_comment(n,NODE_VALUE_STRING_dup(DEMTOP->precache_models[default_modelindex]));
-    }
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_FRAME,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_frame),0));
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_COLORMAP,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_colormap),0));
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_SKIN,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_skin),0));
-    node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ORIGIN, V_FLOAT, H_COORD,
-                    NODE_VALUE_FLOAT_dup(default_origin[0]),
-                    NODE_VALUE_FLOAT_dup(default_origin[1]),
-                    NODE_VALUE_FLOAT_dup(default_origin[2]), 0));
-    node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ANGLES, V_FLOAT, H_ANGLE,
-                    NODE_VALUE_FLOAT_dup(default_angles[0]),
-                    NODE_VALUE_FLOAT_dup(default_angles[1]),
-                    NODE_VALUE_FLOAT_dup(default_angles[2]), 0));
-    return node_init_all(TOKEN_SPAWNSTATIC, H_DEM_SPAWNSTATIC, n, 0);
+  default_modelindex = ReadByte(m);
+  default_frame = ReadByte(m);
+  default_colormap = ReadByte(m);
+  default_skin = ReadByte(m);
+  for (i=0 ; i<3 ; i++) {
+    default_origin[i] = ReadCoord(m);
+    default_angles[i] = ReadAngle(m);
   }
+
+  /* construct node tree and return the root of it */
+  n = node_command_init(TOKEN_DEFAULT_MODELINDEX,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_modelindex),0);
+  if (default_modelindex >= 1 &&
+      default_modelindex <= DEMTOP->nummodels &&
+      DEMTOP->precache_models[default_modelindex][0]!='*') {
+    node_add_comment(n,NODE_VALUE_STRING_dup(DEMTOP->precache_models[default_modelindex]));
+  }
+  node_add_next(n,node_command_init(TOKEN_DEFAULT_FRAME,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_frame),0));
+  node_add_next(n,node_command_init(TOKEN_DEFAULT_COLORMAP,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_colormap),0));
+  node_add_next(n,node_command_init(TOKEN_DEFAULT_SKIN,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_skin),0));
+  node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ORIGIN, V_FLOAT, H_COORD,
+                  NODE_VALUE_FLOAT_dup(default_origin[0]),
+                  NODE_VALUE_FLOAT_dup(default_origin[1]),
+                  NODE_VALUE_FLOAT_dup(default_origin[2]), 0));
+  node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ANGLES, V_FLOAT, H_ANGLE,
+                  NODE_VALUE_FLOAT_dup(default_angles[0]),
+                  NODE_VALUE_FLOAT_dup(default_angles[1]),
+                  NODE_VALUE_FLOAT_dup(default_angles[2]), 0));
+  return node_init_all(TOKEN_SPAWNSTATIC, H_DEM_SPAWNSTATIC, n, 0);
 }
 
 /* 0x15 ----------------------------------------------------------------------*/
@@ -1415,82 +1328,38 @@ node* do_spawnbaseline_message_read_bin(BB_t* m)
 
   int i;
 
-  int data1, data2, data3;
-
-  if (DEMTOP->game == HEXENII) {
-    /* binary in */
-    entity = ReadShort(m);
-    default_modelindex = ReadByte(m);
-    default_modelindex |= (ReadByte(m) << 8);
-    default_frame = ReadByte(m);
-    default_colormap = ReadByte(m);
-    default_skin = ReadByte(m);
-    data1=ReadByte(m);
-    data2=ReadByte(m);
-    data3=ReadByte(m);
-    for (i=0 ; i<3 ; i++) {
-      default_origin[i] = ReadCoord(m);
-      default_angles[i] = ReadAngle(m);
-    }
-
-    /* construct node tree and return the root of it */
-    n = node_command_init(TOKEN_ENTITY,V_INT,H_SHORT,NODE_VALUE_INT_dup(entity),0);
-    tn = node_command_init(TOKEN_DEFAULT_MODELINDEX,V_INT,H_SHORT,NODE_VALUE_INT_dup(default_modelindex),0);
-    if (default_modelindex >= 1 && 
-        default_modelindex <= DEMTOP->nummodels && 
-        DEMTOP->precache_models[default_modelindex][0]!='*') {
-      node_add_comment(tn,NODE_VALUE_STRING_dup(DEMTOP->precache_models[default_modelindex]));
-    }
-    node_add_next(n,tn);
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_FRAME,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_frame),0));
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_COLORMAP,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_colormap),0));
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_SKIN,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_skin),0));
-    node_add_next(n,node_triple_command_init(TOKEN_DATA,V_BYTEHEX,H_BYTE,
-                    NODE_VALUE_INT_dup(data1),NODE_VALUE_INT_dup(data2),NODE_VALUE_INT_dup(data3),0));
-    node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ORIGIN, V_FLOAT, H_COORD,
-                    NODE_VALUE_FLOAT_dup(default_origin[0]),
-                    NODE_VALUE_FLOAT_dup(default_origin[1]),
-                    NODE_VALUE_FLOAT_dup(default_origin[2]), 0));
-    node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ANGLES, V_FLOAT, H_ANGLE,
-                    NODE_VALUE_FLOAT_dup(default_angles[0]),
-                    NODE_VALUE_FLOAT_dup(default_angles[1]),
-                    NODE_VALUE_FLOAT_dup(default_angles[2]), 0));
-    return node_init_all(TOKEN_SPAWNBASELINE, H_DEM_SPAWNBASELINE, n, 0);
+  /* binary in */
+  entity = ReadShort(m);
+  default_modelindex = ReadByte(m);
+  default_frame = ReadByte(m);
+  default_colormap = ReadByte(m);
+  default_skin = ReadByte(m);
+  for (i=0 ; i<3 ; i++) {
+    default_origin[i] = ReadCoord(m);
+    default_angles[i] = ReadAngle(m);
   }
-  else {
-    /* binary in */
-    entity = ReadShort(m);
-    default_modelindex = ReadByte(m);
-    default_frame = ReadByte(m);
-    default_colormap = ReadByte(m);
-    default_skin = ReadByte(m);
-    for (i=0 ; i<3 ; i++) {
-      default_origin[i] = ReadCoord(m);
-      default_angles[i] = ReadAngle(m);
-    }
 
-    /* construct node tree and return the root of it */
-    n = node_command_init(TOKEN_ENTITY,V_INT,H_SHORT,NODE_VALUE_INT_dup(entity),0);
-    tn = node_command_init(TOKEN_DEFAULT_MODELINDEX,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_modelindex),0);
-    if (default_modelindex >= 1 && 
-        default_modelindex <= DEMTOP->nummodels && 
-        DEMTOP->precache_models[default_modelindex][0]!='*') {
-      node_add_comment(tn,NODE_VALUE_STRING_dup(DEMTOP->precache_models[default_modelindex]));
-    }
-    node_add_next(n,tn);
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_FRAME,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_frame),0));
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_COLORMAP,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_colormap),0));
-    node_add_next(n,node_command_init(TOKEN_DEFAULT_SKIN,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_skin),0));
-    node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ORIGIN, V_FLOAT, H_COORD,
-                    NODE_VALUE_FLOAT_dup(default_origin[0]),
-                    NODE_VALUE_FLOAT_dup(default_origin[1]),
-                    NODE_VALUE_FLOAT_dup(default_origin[2]), 0));
-    node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ANGLES, V_FLOAT, H_ANGLE,
-                    NODE_VALUE_FLOAT_dup(default_angles[0]),
-                    NODE_VALUE_FLOAT_dup(default_angles[1]),
-                    NODE_VALUE_FLOAT_dup(default_angles[2]), 0));
-    return node_init_all(TOKEN_SPAWNBASELINE, H_DEM_SPAWNBASELINE, n, 0);
+  /* construct node tree and return the root of it */
+  n = node_command_init(TOKEN_ENTITY,V_INT,H_SHORT,NODE_VALUE_INT_dup(entity),0);
+  tn = node_command_init(TOKEN_DEFAULT_MODELINDEX,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_modelindex),0);
+  if (default_modelindex >= 1 && 
+      default_modelindex <= DEMTOP->nummodels && 
+      DEMTOP->precache_models[default_modelindex][0]!='*') {
+    node_add_comment(tn,NODE_VALUE_STRING_dup(DEMTOP->precache_models[default_modelindex]));
   }
+  node_add_next(n,tn);
+  node_add_next(n,node_command_init(TOKEN_DEFAULT_FRAME,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_frame),0));
+  node_add_next(n,node_command_init(TOKEN_DEFAULT_COLORMAP,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_colormap),0));
+  node_add_next(n,node_command_init(TOKEN_DEFAULT_SKIN,V_INT,H_BYTE,NODE_VALUE_INT_dup(default_skin),0));
+  node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ORIGIN, V_FLOAT, H_COORD,
+                  NODE_VALUE_FLOAT_dup(default_origin[0]),
+                  NODE_VALUE_FLOAT_dup(default_origin[1]),
+                  NODE_VALUE_FLOAT_dup(default_origin[2]), 0));
+  node_add_next(n,node_triple_command_init(TOKEN_DEFAULT_ANGLES, V_FLOAT, H_ANGLE,
+                  NODE_VALUE_FLOAT_dup(default_angles[0]),
+                  NODE_VALUE_FLOAT_dup(default_angles[1]),
+                  NODE_VALUE_FLOAT_dup(default_angles[2]), 0));
+  return node_init_all(TOKEN_SPAWNBASELINE, H_DEM_SPAWNBASELINE, n, 0);
 }
 
 /* 0x17 ----------------------------------------------------------------------*/
@@ -1615,7 +1484,7 @@ node* do_signonnum_message_read_bin(BB_t* m)
 node* do_centerprint_message_read_bin(BB_t* m)
 {
   /* variables */
-  char text[1000];
+  char text[MAX_STRING_SIZE];
 
   /* binary in */
   ReadString(m, text);
@@ -1666,24 +1535,14 @@ node* do_spawnstaticsound_message_read_bin(BB_t* m)
 
   /* binary in */
   for (i=0 ; i<3 ; i++) origin[i] = ReadCoord(m);
-  if (DEMTOP->game==HEXENII) {
-    soundnum = ReadShort(m);
-  }
-  else {
-    soundnum = ReadByte(m);
-  }
+  soundnum = ReadByte(m);
   vol = (float) ReadByte(m) / 255.0;
   attenuation = (float) ReadByte(m) / 64.0;
 
   /* construct node tree and return the root of it */
   n = node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
       NODE_VALUE_FLOAT_dup(origin[0]),NODE_VALUE_FLOAT_dup(origin[1]),NODE_VALUE_FLOAT_dup(origin[2]),0);
-  if (DEMTOP->game==HEXENII) {
-    tn = node_command_init(TOKEN_SOUNDNUM,V_INT,H_SHORT,NODE_VALUE_INT_dup(soundnum),0);
-  }
-  else {
-    tn = node_command_init(TOKEN_SOUNDNUM,V_INT,H_BYTE,NODE_VALUE_INT_dup(soundnum),0);
-  }    
+  tn = node_command_init(TOKEN_SOUNDNUM,V_INT,H_BYTE,NODE_VALUE_INT_dup(soundnum),0);
   if (soundnum >= 1 &&
       soundnum <= DEMTOP->numsounds) {
     node_add_comment(tn,NODE_VALUE_STRING_dup(DEMTOP->precache_sounds[soundnum]));
@@ -1705,7 +1564,7 @@ node* do_intermission_message_read_bin(void)
 node* do_finale_message_read_bin(BB_t* m)
 {
   /* variables */
-  char text[1000];
+  char text[MAX_STRING_SIZE];
 
   /* binary in */
   ReadString(m, text);
@@ -1747,7 +1606,7 @@ node* do_sellscreen_message_read_bin(void)
 node* do_cutscene_message_read_bin(BB_t* m)
 {
   /* variables */
-  char text[1000];
+  char text[MAX_STRING_SIZE];
 
   /* binary in */
   ReadString(m, text);
@@ -1757,255 +1616,6 @@ node* do_cutscene_message_read_bin(BB_t* m)
                        H_SIMPLE);
 }
 
-/* Hexen II only part */
-
-/* 0x22 ----------------------------------------------------------------------*/
-node* do_particle2_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_PARTICLE2,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x23 ----------------------------------------------------------------------*/
-node* do_hexen_cutscene_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_CUTSCENE,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x24 ----------------------------------------------------------------------*/
-node* do_midi_name_message_read_bin(BB_t* m)
-{
-  /* variables */
-  char text[1000];
-
-  /* binary in */
-  ReadString(m, text);
-
-  /* construct node tree and return the root of it */
-  return node_add_hint(node_command_init(TOKEN_MIDI_NAME,V_STRING,H_STRING,NODE_VALUE_STRING_dup(text),0),
-                       H_SIMPLE);
-}
-
-/* 0x25 ----------------------------------------------------------------------*/
-node* do_updateclass_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  /* variables */
-  long player;
-  long class;
-
-  /* binary in */
-  player = ReadByte(m);
-  class = ReadByte(m);
-
-  /* construct node tree */
-  node_add_next(
-    tn = node_command_init(TOKEN_PLAYER,V_INT,H_BYTE,NODE_VALUE_INT_dup(player),0),
-    node_command_init(TOKEN_CLASS,V_INT,H_BYTE,NODE_VALUE_INT_dup(class),0)
-  );
-  n = node_init(TOKEN_UPDATECLASS,tn,0);
-  n->hint=H_SIMPLE;
-
-  /* return node */
-  return n;
-}
-
-/* 0x26 ----------------------------------------------------------------------*/
-node* do_particle3_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_PARTICLE3,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x27 ----------------------------------------------------------------------*/
-node* do_particle4_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_PARTICLE4,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x28 ----------------------------------------------------------------------*/
-node* do_set_view_flags_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_SET_VIEW_FLAGS,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x29 ----------------------------------------------------------------------*/
-node* do_clear_view_flags_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_CLEAR_VIEW_FLAGS,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x2A ----------------------------------------------------------------------*/
-node* do_start_effect_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_START_EFFECT,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x2B ----------------------------------------------------------------------*/
-node* do_end_effect_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_END_EFFECT,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x2C ----------------------------------------------------------------------*/
-node* do_plaque_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_PLAQUE,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x2D ----------------------------------------------------------------------*/
-node* do_particle_explosion_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_PARTICLE_EXPLOSION,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x2E ----------------------------------------------------------------------*/
-node* do_set_view_tint_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_SET_VIEW_TINT,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x2F ----------------------------------------------------------------------*/
-node* do_reference_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_REFERENCE,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
-/* 0x30 ----------------------------------------------------------------------*/
-node* do_clear_edicts_message_read_bin(BB_t* m)
-{
-  /* variables */
-  long edict;
-
-  /* binary in */
-  edict = ReadByte(m);
-
-  /* construct node tree and return the root of it */
-  return node_command_init(TOKEN_CLEAR_EDICTS,V_INT,H_BYTE,NODE_VALUE_INT_dup(edict),0);
-}
-
-/* 0x31 ----------------------------------------------------------------------*/
-node* do_update_inv_message_read_bin(BB_t* m)
-{
-  node *n, *tn;
-
-  n=NULL;
-  if (m->p < m->end) {
-    for (tn=NULL; m->p < m->end;) {
-      tn=node_link(tn,node_init_all(V_BYTEHEX,H_SIMPLE,NODE_VALUE_BYTEHEX_dup((unsigned char)ReadByte(m)),0));
-    }
-    n=node_init_all(TOKEN_DATA,H_SIMPLE,tn,0);
-  }
-  return node_add_comment(node_init_all(TOKEN_UPDATE_INV,H_SIMPLE,n,0),NODE_VALUE_STRING_dup("unknown message structure"));
-}
-
 /* >= 0x80 -------------------------------------------------------------------*/
 node* do_updateentity_message_read_bin(BB_t* m, long mask)
 {
@@ -2020,7 +1630,7 @@ node* do_updateentity_message_read_bin(BB_t* m, long mask)
   long effects;
   vec3_t origin;
   vec3_t angles;
-  long new;
+  long new_;
 
   /* init */
   origin[0]=origin[1]=origin[2]=angles[0]=angles[1]=angles[2]=0.0;
@@ -2039,7 +1649,7 @@ node* do_updateentity_message_read_bin(BB_t* m, long mask)
   angles[1] = mask & 0x0010 ? ReadAngle(m) : 0;
   origin[2] = mask & 0x0008 ? ReadCoord(m) : 0;
   angles[2] = mask & 0x0200 ? ReadAngle(m) : 0;
-  new = mask & 0x0020 ? 1 : 0;
+  new_ = mask & 0x0020 ? 1 : 0;
 
   /* construct node tree and return the root of it */
   n=node_command_init(TOKEN_ENTITY,V_INT,H_SHORT,NODE_VALUE_INT_dup(entity),0);
@@ -2082,7 +1692,7 @@ node* do_updateentity_message_read_bin(BB_t* m, long mask)
   if (mask&0x0200) { 
     node_add_next(n,node_command_init(TOKEN_ANGLES_3,V_FLOAT,H_ANGLE,NODE_VALUE_FLOAT_dup(angles[2]),0));
   }
-  if (new) {
+  if (new_) {
     node_add_next(n,node_init_all(TOKEN_NEW,H_SIMPLE,NULL,0));
   }
   return node_init_all(TOKEN_UPDATEENTITY,H_DEM_UPDATEENTITY,n,0);
@@ -2098,10 +1708,14 @@ node* do_updateentity_message_read_bin(BB_t* m, long mask)
 
 node* DEM_block_write_text(node* b)
 {
-  /* type is TOKEN_TRACK or */
+  /* b may be NULL (something was wrong or removed) */
+  if (b == NULL) return b;
+  /* type is TOKEN_TRACK or TOKEN_*/
   /* down is camera, down->next is the next message */
-  if (b->type==TOKEN_TRACK || b->down->next!=NULL) 
+  if ( ( b->type==TOKEN_TRACK ) || 
+       ( b->type==TOKEN_BLOCK && b->down->next!=NULL ) ) {
     node_write_text(b,0);
+  }
   return b;
 }
 
@@ -2114,9 +1728,10 @@ node* DEM_block_write_bin(node* b)
   BB_t m;
   node* tn;
   node* tn1;
-  long datasize;
+  long datasize = 0;
 
-  datasize = 0;
+  /* b may be NULL (something was wrong or removed) */
+  if (b == NULL) return b;
 
   /* we start from the beginning */
   m.p = m.buffer;
@@ -2194,7 +1809,7 @@ node* DEM_block_write_bin(node* b)
   } 
   
   if (datasize) { /* if there was something after camera */
-    if (fwrite(m.buffer, 1, datasize, output_file) != datasize) 
+    if (fwrite(m.buffer, 1, datasize, output_file) != (size_t)datasize) 
       syserror(FIWRITE, output_filename);
     if (fflush(output_file) != 0) 
       syserror(errno, output_filename);
@@ -2373,12 +1988,12 @@ void do_dem_clientdata_message_write_bin(node* n, BB_t* m)
   WriteShort(m,mask); /* mask */
 
   if (mask&0x0001) { /* view_ofs_z */
-    WriteChar(m,*(float*)c->down->down);
+    WriteChar(m,(char)(*(float*)c->down->down));
     c=c->next;
   }
 
   if (mask&0x0002) { /* punchangle_x */
-    WriteChar(m,*(float*)c->down->down);
+    WriteChar(m,(char)(*(float*)c->down->down));
     c=c->next;
   }
 
@@ -2412,12 +2027,12 @@ void do_dem_clientdata_message_write_bin(node* n, BB_t* m)
     c=c->next;
   }
 
-  if (mask&0x0004) WriteChar(m,angles_1); 
-  if (mask&0x0020) WriteChar(m,vel_x); 
-  if (mask&0x0008) WriteChar(m,angles_2); 
-  if (mask&0x0040) WriteChar(m,vel_y); 
-  if (mask&0x0010) WriteChar(m,angles_3); 
-  if (mask&0x0080) WriteChar(m,vel_z); 
+  if (mask&0x0004) WriteChar(m,(char)angles_1); 
+  if (mask&0x0020) WriteChar(m,(char)vel_x); 
+  if (mask&0x0008) WriteChar(m,(char)angles_2); 
+  if (mask&0x0040) WriteChar(m,(char)vel_y); 
+  if (mask&0x0010) WriteChar(m,(char)angles_3); 
+  if (mask&0x0080) WriteChar(m,(char)vel_z); 
 
   if (mask&0x0200) { /* items (many poss.) */
     WriteLong(m,items);
