@@ -1,5 +1,5 @@
 /*  lmpc -- the Little Movie Processing Centre
-    Copyright (C) 1994-98 Uwe Girlich
+    Copyright (C) 1994-99 Uwe Girlich
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
     Uwe Girlich
-    Erika-von-Brockdorff-Strasse 2
+    Dantestrasse 20
     04159 Leipzig
     Deutschland / Germany
     E-mail: Uwe.Girlich@itp.uni-leipzig.de */
@@ -35,7 +35,7 @@
 
 
 cs_t cs_info[] = {
-  { DM2_CS_NAME,		"player name" },
+  { DM2_CS_NAME,		"map name" },
   { DM2_CS_CDTRACK,		"CD track" },
   { DM2_CS_SKY,			"sky" },
   { DM2_CS_SKYAXIS,		"skyaxis" },
@@ -104,6 +104,7 @@ token_t DM2_token[]={
 	{ "muzzleflash",		TOKEN_MUZZLEFLASH,		0x01 },
 	{ "bad",			TOKEN_BAD,			0x00 },
 
+	{ "unknown_mask_bits",		TOKEN_UNKNOWN_MASK_BITS,	0x00 },
 	{ "serverversion",		TOKEN_SERVERVERSION,		0x00 },
 	{ "mapname",			TOKEN_MAPNAME,			0x00 },
 	{ "index",			TOKEN_INDEX,			0x00 },
@@ -111,9 +112,6 @@ token_t DM2_token[]={
 	{ "seq1",			TOKEN_SEQ1,			0x00 },
 	{ "seq2",			TOKEN_SEQ2,			0x00 },
 	{ "origin",			TOKEN_ORIGIN,			0x00 },
-	{ "mask",			TOKEN_MASK,			0x00 },
-	{ "data2",			TOKEN_DATA2,			0x00 },
-	{ "mask2",			TOKEN_MASK2,			0x00 },
 	{ "entity",			TOKEN_ENTITY,			0x00 },
 	{ "trace_endpos",		TOKEN_TRACE_ENDPOS,		0x00 },
 	{ "movedir",			TOKEN_MOVEDIR,			0x00 },
@@ -168,11 +166,23 @@ token_t DM2_token[]={
 	{ "stats",			TOKEN_STATS,			0x00 },
 	{ "packetentity",		TOKEN_PACKETENTITY,		0x00 },
 	{ "remove",			TOKEN_REMOVE,			0x00 },
-	{ "offset",			TOKEN_OFFSET,			0x00 },
 	{ "areas",			TOKEN_AREAS,			0x00 },
 	{ "isdemo",			TOKEN_ISDEMO,			0x00 },
 	{ "size",			TOKEN_SIZE,			0x00 },
-	{ "percent",			TOKEN_PERCENT,			0x00 }
+	{ "percent",			TOKEN_PERCENT,			0x00 },
+	{ "style",			TOKEN_STYLE,			0x00 },
+	{ "dest_entity",		TOKEN_DEST_ENTITY,		0x00 },
+	{ "nextid",			TOKEN_NEXTID,			0x00 },
+	{ "start",			TOKEN_START,			0x00 },
+	{ "dest_origin",		TOKEN_DEST_ORIGIN,		0x00 },
+	{ "type",			TOKEN_TYPE,			0x00 },
+	{ "flash_entity",		TOKEN_FLASH_ENTITY,		0x00 },
+	{ "pos1",			TOKEN_POS1,			0x00 },
+	{ "pos2",			TOKEN_POS2,			0x00 },
+	{ "pos3",			TOKEN_POS3,			0x00 },
+	{ "pos4",			TOKEN_POS4,			0x00 },
+	{ "plat2flags",			TOKEN_PLAT2FLAGS,		0x00 },
+	{ "wait",			TOKEN_WAIT,			0x00 }
 };
 
 void udm2_init(void)
@@ -215,7 +225,7 @@ unsigned long isDM2bin(char *filename)
   filesize=buf.st_size;
   rewind(file);
   tics=0;
-  while (ftell(file)<filesize) {
+  while (ftell(file)<(long)filesize) {
     tics++;
     if (fread(buffer, 1, 4, file)!=4) return 0;
     blocksize = ((buffer[0])    ) +
@@ -225,7 +235,7 @@ unsigned long isDM2bin(char *filename)
     if (blocksize==0xFFFFFFFF) break; /* not necessary in q2test */
     if (fseek(file, blocksize, SEEK_CUR)!=0) return 0;
   }
-  if (ftell(file)!=filesize) return 0;
+  if (ftell(file)!=(long)filesize) return 0;
   fclose(file);
   return tics;
 }
@@ -240,8 +250,8 @@ int isDM2txt(char* filename)
   */
   #define BUFFER_SIZE 1000
   char buffer[BUFFER_SIZE];
-  unsigned long blocksize;
-  int p;
+  long blocksize;
+  long p;
   int state;
 
   if ((file=fopen(filename, "rb"))==NULL) return 0;
@@ -336,13 +346,13 @@ void DM2_block_read_bin(DM2_t *d, BB_t* m)
   m->p = m->buffer;
   m->end = &(m->buffer[4]);
   m->size = ReadLong(m);
-  if (m->size == 0xFFFFFFFF) { /* end block */
+  if (m->size == (long)0xFFFFFFFF) { /* end block */
     m->size = 0;
   }
   else {
     if (m->size > BBB_MAX)
       syserror(WDM2,d->filename);
-    if (fread(m->buffer+4,1,m->size,d->file)!=m->size) 
+    if (fread(m->buffer+4,1,m->size,d->file)!=(size_t)m->size) 
       syserror(FIREAD,d->filename);
   }  
   m->size+=4;
@@ -352,13 +362,15 @@ void DM2_block_read_bin(DM2_t *d, BB_t* m)
 }
 
 #define DM2TOP ((DM2_t*)(m->top))
-#define MODELNAME(x) ((x)>=1 && (x)<DM2_MAX_MODELS  ? DM2TOP->config_string[DM2_CS_MODELS      + (x)] : NULL)
-#define SOUNDNAME(x) ((x)>=1 && (x)<DM2_MAX_SOUNDS  ? DM2TOP->config_string[DM2_CS_SOUNDS      + (x)] : NULL)
-#define IMAGENAME(x) ((x)>=1 && (x)<DM2_MAX_IMAGES  ? DM2TOP->config_string[DM2_CS_IMAGES      + (x)] : NULL)
-#define LIGHTNAME(x) ((x)>=1 && (x)<DM2_MAX_LIGHTS  ? DM2TOP->config_string[DM2_CS_LIGHTS      + (x)] : NULL) 
-#define ITEMNAME(x)  ((x)>=1 && (x)<DM2_MAX_ITEMS   ? DM2TOP->config_string[DM2_CS_ITEMS       + (x)] : NULL)
-#define SKINNAME(x)  ((x)>=1 && (x)<DM2_MAX_CLIENTS ? DM2TOP->config_string[DM2_CS_PLAYERSKINS + (x)] : NULL)
+#define MODELNAME(x) ((x)>=1 && (x)<DM2_MAX_MODELS  ? DM2TOP->config_string[DM2_CS_MODELS      + (x)] : (char*)NULL)
+#define SOUNDNAME(x) ((x)>=1 && (x)<DM2_MAX_SOUNDS  ? DM2TOP->config_string[DM2_CS_SOUNDS      + (x)] : (char*)NULL)
+#define IMAGENAME(x) ((x)>=1 && (x)<DM2_MAX_IMAGES  ? DM2TOP->config_string[DM2_CS_IMAGES      + (x)] : (char*)NULL)
+#define LIGHTNAME(x) ((x)>=1 && (x)<DM2_MAX_LIGHTS  ? DM2TOP->config_string[DM2_CS_LIGHTS      + (x)] : (char*)NULL) 
+#define ITEMNAME(x)  ((x)>=1 && (x)<DM2_MAX_ITEMS   ? DM2TOP->config_string[DM2_CS_ITEMS       + (x)] : (char*)NULL)
+#define SKINNAME(x)  ((x)>=1 && (x)<DM2_MAX_CLIENTS ? DM2TOP->config_string[DM2_CS_PLAYERSKINS + (x)] : (char*)NULL)
+/*
 #define SERVERVERSION (DM2TOP->serverversion)
+*/
 #define DEMOTYPE (DM2TOP->demotype)
 
 node* DM2_bin_to_node(BB_t *m, int opt)
@@ -379,8 +391,13 @@ node* DM2_bin_to_node(BB_t *m, int opt)
     return n;
   }
 
+  /*
+  fprintf(stderr,"block start, base=0x%08lX, end=0x%08lX\n",
+                  m->base + (m->p-m->buffer),
+                  m->base + (m->p-m->buffer) + size - 1);
+  */
   while (m->p < m->end) {
-    /* fprintf(stderr, "block in, pos=0x%08lX\n", m->base + (m->p-m->buffer)); */
+    /* fprintf(stderr, "pos=0x%08lX\n", m->base + (m->p-m->buffer)); */ 
     if (opt & 0x01) {
       tn=node_init_all(GEN_NOTHING,H_NOTHING,NULL,0);
       sprintf(ts, "0x%08lX", m->base + (m->p-m->buffer));
@@ -388,7 +405,7 @@ node* DM2_bin_to_node(BB_t *m, int opt)
       n=node_link(n,tn);
     }
     id = ReadByte(m);
-    /* fprintf(stderr,"id=0x%02lX\n", id); */
+    /* fprintf(stderr,"id=0x%02lX\n", id); */ 
     switch(id) {
       case 0x00: { /* bad */
         /* node tree */
@@ -431,202 +448,15 @@ node* DM2_bin_to_node(BB_t *m, int opt)
       break;
       case 0x03: { /* temp_entity */
         /* variables */
-        long entitytype;
-        vec3_t origin;
-        vec3_t trace_endpos;
-        vec3_t offset;
-        vec3_t movedir;
-        long entity=0;
-        long sound=0;
-        long color=0;
-        long count=0;
+        #include "udm2-te1.c"
         
         /* binary in */
-        entitytype = ReadByte(m);
-        switch (entitytype) {
-          /* point entity */
-          case DM2_TE_EXPLOSION1:
-          case DM2_TE_EXPLOSION2:
-          case DM2_TE_ROCKET_EXPLOSION:
-          case DM2_TE_GRENADE_EXPLOSION:
-          case DM2_TE_ROCKET_EXPLOSION_WATER:
-          case DM2_TE_GRENADE_EXPLOSION_WATER:
-          case DM2_TE_BFG_EXPLOSION:
-          case DM2_TE_BFG_BIGEXPLOSION:
-          case DM2_TE_BOSSTPORT:
-          case DM2_TE_28:
-            ReadPosition(m, origin);
-          break;
-          /* impact entity */
-          case DM2_TE_GUNSHOT:
-          case DM2_TE_BLOOD:
-          case DM2_TE_BLASTER:
-          case DM2_TE_SHOTGUN:
-          case DM2_TE_SPARKS:
-          case DM2_TE_SCREEN_SPARKS:
-          case DM2_TE_SHIELD_SPARKS:
-          case DM2_TE_BULLET_SPARKS:
-          case DM2_TE_GREENBLOOD:
-            ReadPosition(m, origin);
-            ReadDir(m, movedir);
-          break;
-          /* line entity */
-          case DM2_TE_RAILTRAIL:
-          case DM2_TE_BUBBLETRAIL:
-          case DM2_TE_BFG_LASER:
-          case DM2_TE_PLASMATRAIL:
-            ReadPosition(m, origin);
-            ReadPosition(m, trace_endpos);
-          break;
-          /* special entity */
-          case DM2_TE_SPLASH:
-            count = ReadByte(m);
-            ReadPosition(m, origin);
-            ReadDir(m, movedir);
-            sound = ReadByte(m);
-          break;
-          case DM2_TE_LASER_SPARKS:
-          case DM2_TE_WELDING_SPARKS:
-          case DM2_TE_29:
-            count = ReadByte(m);
-            ReadPosition(m, origin);
-            ReadDir(m, movedir);
-            color = ReadByte(m);
-          break;
-          case DM2_TE_PARASITE_ATTACK:
-          case DM2_TE_MEDIC_CABLE_ATTACK:
-            entity = ReadShort(m);
-            ReadPosition(m, origin);
-            ReadPosition(m, trace_endpos);
-          break;
-          case DM2_TE_GRAPPLE_CABLE:
-            entity = ReadShort(m);
-            ReadPosition(m, origin);
-            ReadPosition(m, trace_endpos);
-            ReadPosition(m, offset);
-          break;
-          default:
-            syserror(WDM2, DM2TOP->filename);
-          break;
-        }
+        #include "udm2-te2.c"
+
+        tn = NULL;
+        /* construct node tree */
+        #include "udm2-te3.c"
 
-        tn = NULL;              
-        tn = node_link(tn,node_command_init(TOKEN_ENTITYTYPE,V_INT,H_BYTE,
-                          NODE_VALUE_INT_dup(entitytype),0));
-        switch (entitytype) {
-          /* point entity */
-          case DM2_TE_EXPLOSION1:
-          case DM2_TE_EXPLOSION2:
-          case DM2_TE_ROCKET_EXPLOSION:
-          case DM2_TE_GRENADE_EXPLOSION:
-          case DM2_TE_ROCKET_EXPLOSION_WATER:
-          case DM2_TE_GRENADE_EXPLOSION_WATER:
-          case DM2_TE_BFG_EXPLOSION:
-          case DM2_TE_BFG_BIGEXPLOSION:
-          case DM2_TE_BOSSTPORT:
-          case DM2_TE_28:
-            tn=node_link(tn,node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(origin[0]),
-                            NODE_VALUE_FLOAT_dup(origin[1]),
-                            NODE_VALUE_FLOAT_dup(origin[2]),0));
-          break;
-          /* impact entity */
-          case DM2_TE_GUNSHOT:
-          case DM2_TE_BLOOD:
-          case DM2_TE_BLASTER:
-          case DM2_TE_SHOTGUN:
-          case DM2_TE_SPARKS:
-          case DM2_TE_SCREEN_SPARKS:
-          case DM2_TE_SHIELD_SPARKS:
-          case DM2_TE_BULLET_SPARKS:
-          case DM2_TE_GREENBLOOD:
-            tn=node_link(tn,node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(origin[0]),
-                            NODE_VALUE_FLOAT_dup(origin[1]),
-                            NODE_VALUE_FLOAT_dup(origin[2]),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_MOVEDIR,V_FLOAT,H_DIR,
-                            NODE_VALUE_FLOAT_dup(movedir[0]),
-                            NODE_VALUE_FLOAT_dup(movedir[1]),
-                            NODE_VALUE_FLOAT_dup(movedir[2]),0));
-          break;
-          /* line entity */
-          case DM2_TE_RAILTRAIL:
-          case DM2_TE_BUBBLETRAIL:
-          case DM2_TE_BFG_LASER:
-          case DM2_TE_PLASMATRAIL:
-            tn=node_link(tn,node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(origin[0]),
-                            NODE_VALUE_FLOAT_dup(origin[1]),
-                            NODE_VALUE_FLOAT_dup(origin[2]),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_TRACE_ENDPOS,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(trace_endpos[0]),
-                            NODE_VALUE_FLOAT_dup(trace_endpos[1]),
-                            NODE_VALUE_FLOAT_dup(trace_endpos[2]),0));
-          break;
-          /* special entity */
-          case DM2_TE_SPLASH:
-            tn=node_link(tn,node_command_init(TOKEN_COUNT,V_INT,H_BYTE,
-                            NODE_VALUE_INT_dup(count),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(origin[0]),
-                            NODE_VALUE_FLOAT_dup(origin[1]),
-                            NODE_VALUE_FLOAT_dup(origin[2]),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_MOVEDIR,V_FLOAT,H_DIR,
-                            NODE_VALUE_FLOAT_dup(movedir[0]),
-                            NODE_VALUE_FLOAT_dup(movedir[1]),
-                            NODE_VALUE_FLOAT_dup(movedir[2]),0));
-            tn=node_link(tn,node_command_init(TOKEN_SOUND,V_INT,H_BYTE,
-                            NODE_VALUE_INT_dup(sound),0));
-          break;
-          case DM2_TE_LASER_SPARKS:
-          case DM2_TE_WELDING_SPARKS:
-          case DM2_TE_29:
-            tn=node_link(tn,node_command_init(TOKEN_COUNT,V_INT,H_BYTE,
-                            NODE_VALUE_INT_dup(count),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(origin[0]),
-                            NODE_VALUE_FLOAT_dup(origin[1]),
-                            NODE_VALUE_FLOAT_dup(origin[2]),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_MOVEDIR,V_FLOAT,H_DIR,
-                            NODE_VALUE_FLOAT_dup(movedir[0]),
-                            NODE_VALUE_FLOAT_dup(movedir[1]),
-                            NODE_VALUE_FLOAT_dup(movedir[2]),0));
-            tn=node_link(tn,node_command_init(TOKEN_COLOR,V_INT,H_BYTE,
-                            NODE_VALUE_INT_dup(color),0));
-          break;
-          case DM2_TE_PARASITE_ATTACK:
-          case DM2_TE_MEDIC_CABLE_ATTACK:
-            tn=node_link(tn,node_command_init(TOKEN_ENTITY,V_INT,H_SHORT,
-                            NODE_VALUE_INT_dup(entity),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(origin[0]),
-                            NODE_VALUE_FLOAT_dup(origin[1]),
-                            NODE_VALUE_FLOAT_dup(origin[2]),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_TRACE_ENDPOS,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(trace_endpos[0]),
-                            NODE_VALUE_FLOAT_dup(trace_endpos[1]),
-                            NODE_VALUE_FLOAT_dup(trace_endpos[2]),0));
-          break;
-          case DM2_TE_GRAPPLE_CABLE:
-            tn=node_link(tn,node_command_init(TOKEN_ENTITY,V_INT,H_SHORT,
-                            NODE_VALUE_INT_dup(entity),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(origin[0]),
-                            NODE_VALUE_FLOAT_dup(origin[1]),
-                            NODE_VALUE_FLOAT_dup(origin[2]),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_TRACE_ENDPOS,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(trace_endpos[0]),
-                            NODE_VALUE_FLOAT_dup(trace_endpos[1]),
-                            NODE_VALUE_FLOAT_dup(trace_endpos[2]),0));
-            tn=node_link(tn,node_triple_command_init(TOKEN_OFFSET,V_FLOAT,H_COORD,
-                            NODE_VALUE_FLOAT_dup(offset[0]),
-                            NODE_VALUE_FLOAT_dup(offset[1]),
-                            NODE_VALUE_FLOAT_dup(offset[2]),0));
-          break;
-          default:
-            syserror(WDM2, DM2TOP->filename);
-          break;
-        }
         n=node_link(n,node_init_all(TOKEN_TEMP_ENTITY,H_SIMPLE,tn,0));
       }
       break;
@@ -695,10 +525,10 @@ node* DM2_bin_to_node(BB_t *m, int opt)
 
         mask = ReadByte(m);
         soundnum = ReadByte(m);
-        vol = (mask & 0x01) ? ((float)ReadByte(m) / 255.0) : (1.0);
-        attenuation = (mask & 0x02) ? ((float)ReadByte(m) / 64.0) : (1.0);
-        timeofs = (mask & 0x10) ? ((float)ReadByte(m) * 0.001) : (0.0);
-        if (mask & 0x08) {
+        vol = (mask & MASK_DM2_SOUND_VOL) ? ((float)ReadByte(m) / 255.0) : (1.0);
+        attenuation = (mask & MASK_DM2_SOUND_ATTENUATION) ? ((float)ReadByte(m) / 64.0) : (1.0);
+        timeofs = (mask & MASK_DM2_SOUND_TIMEOFS) ? ((float)ReadByte(m) * 0.001) : (0.0);
+        if (mask & MASK_DM2_SOUND_ENTITY) {
           mix = ReadShort(m);
           channel = mix & 0x07;
           entity = (mix >> 3);
@@ -706,12 +536,23 @@ node* DM2_bin_to_node(BB_t *m, int opt)
           channel = 0;
           entity = 0;
         }
-        if (mask & 0x04) {
+        if (mask & MASK_DM2_SOUND_ORIGIN) {
           ReadPosition(m,origin);
         }
 
         /* node tree */
         tn=NULL;
+
+        if (mask & MASK_DM2_SOUND_UNKNOWNBITS) 
+          tn = node_link(
+                 tn,
+                 node_init(
+                   TOKEN_UNKNOWN_MASK_BITS, 
+                   bitlist_to_nodes(mask & MASK_DM2_SOUND_UNKNOWNBITS),
+                   0
+                 )
+               );
+
         cn = node_command_init(TOKEN_SOUNDNUM, V_INT, H_BYTE,
                                NODE_VALUE_INT_dup(soundnum),0);
         if (soundnum >= 1 &&
@@ -720,25 +561,25 @@ node* DM2_bin_to_node(BB_t *m, int opt)
           node_add_comment(cn, NODE_VALUE_STRING_dup(SOUNDNAME(soundnum)));
         }
         tn = node_link(tn,cn);
-        if (mask & 0x01) {
+        if (mask & MASK_DM2_SOUND_VOL) {
           tn=node_link(tn,node_command_init(TOKEN_VOL,V_FLOAT,H_VOL,
                        NODE_VALUE_FLOAT_dup(vol),0));
         }
-        if (mask & 0x02) {
+        if (mask & MASK_DM2_SOUND_ATTENUATION) {
           tn=node_link(tn,node_command_init(TOKEN_ATTENUATION,V_FLOAT,H_ATTENUATION,
                        NODE_VALUE_FLOAT_dup(attenuation),0));
         }
-        if (mask & 0x10) {
+        if (mask & MASK_DM2_SOUND_TIMEOFS) {
           tn=node_link(tn,node_command_init(TOKEN_TIMEOFS,V_FLOAT,H_TIMEOFS,
                        NODE_VALUE_FLOAT_dup(timeofs),0));
         }
-        if (mask & 0x08) {
+        if (mask & MASK_DM2_SOUND_ENTITY) {
           tn=node_link(tn,node_command_init(TOKEN_CHANNEL,V_INT,0,
                        NODE_VALUE_INT_dup(channel),0));
           tn=node_link(tn,node_command_init(TOKEN_ENTITY,V_INT,0,
                        NODE_VALUE_INT_dup(entity),0));
         }
-        if (mask & 0x04) {
+        if (mask & MASK_DM2_SOUND_ORIGIN) {
           tn=node_link(tn,node_triple_command_init(TOKEN_ORIGIN,V_FLOAT,H_COORD,
                           NODE_VALUE_FLOAT_dup(origin[0]),
                           NODE_VALUE_FLOAT_dup(origin[1]),
@@ -804,8 +645,11 @@ node* DM2_bin_to_node(BB_t *m, int opt)
         ReadString(m, mapname);
 
 	/* memorize the protocol version */
+	/* in a global variable: bad */
         SERVERVERSION = serverversion;
-
+        /* in the DM2 structure: not usable any more */
+        (DM2TOP->serverversion) = serverversion;
+        
 	/* memorize the demo type */
 	if (isdemo) {
 	  DEMOTYPE = isdemo;
@@ -892,8 +736,8 @@ node* DM2_bin_to_node(BB_t *m, int opt)
         long effects=0;
         long renderfx=0;
         long solid=0;
-        long sound = 0;
-        long event = 0;
+        long sound=0;
+        long event=0;
 
         /* binary in */
         mask = ReadByte(m);
@@ -915,7 +759,7 @@ node* DM2_bin_to_node(BB_t *m, int opt)
             if (mask & MASK_DM2_PACKET_SKIN_SHORT) skin = ReadShort(m);
             if (mask & MASK_DM2_PACKET_SKIN_BYTE) skin = ReadByte(m);
           }
-          vwep = skin >> 8;
+          vwep = (skin >> 8) & 0xFFFFFF;
           skin &= 0xFF;
         }
         if (mask & 0x00004000) {
@@ -945,7 +789,17 @@ node* DM2_bin_to_node(BB_t *m, int opt)
 
         /* node tree */
         tn = NULL;
-       
+        
+        if (mask & MASK_DM2_PACKET_UNKNOWNBITS) 
+          tn = node_link(
+                 tn,
+                 node_init(
+                   TOKEN_UNKNOWN_MASK_BITS, 
+                   bitlist_to_nodes(mask & MASK_DM2_PACKET_UNKNOWNBITS),
+                   0
+                 )
+               );
+
         tn = node_link(tn,node_command_init(TOKEN_ENTITY, V_INT, H_SB, 
                                          NODE_VALUE_INT_dup(entity),0));
         if (mask & 0x00000800) {
@@ -1318,9 +1172,9 @@ node* DM2_bin_to_node(BB_t *m, int opt)
         long effects=0;
         long renderfx=0;
         long solid=0;
-        long sound = 0;
-        long event = 0;
-
+        long sound=0;
+        long event=0;
+	
         /* binary in & node tree */
         ttn=NULL;
         for (;;) {
@@ -1347,7 +1201,7 @@ node* DM2_bin_to_node(BB_t *m, int opt)
               if (mask & MASK_DM2_PACKET_SKIN_SHORT) skin = ReadShort(m);
               if (mask & MASK_DM2_PACKET_SKIN_BYTE) skin = ReadByte(m);
             }
-            vwep = skin >> 8;
+            vwep = (skin >> 8) & 0xFFFFFF;
             skin &= 0xFF;
           }
           if (mask & 0x00004000) {
@@ -1374,10 +1228,20 @@ node* DM2_bin_to_node(BB_t *m, int opt)
           if (mask & 0x04000000) sound = ReadByte(m);
           event = (mask & 0x00000020) ? ReadByte(m) : 0;
           if (mask & 0x08000000) solid = ReadShort(m);
-
+          
           /* node tree */
           tn = NULL;
          
+          if (mask & MASK_DM2_PACKET_UNKNOWNBITS) 
+            tn = node_link(
+                   tn,
+                   node_init(
+                     TOKEN_UNKNOWN_MASK_BITS, 
+                     bitlist_to_nodes(mask & MASK_DM2_PACKET_UNKNOWNBITS),
+                     0
+                   )
+                 );
+
           tn = node_link(tn,node_command_init(TOKEN_ENTITY, V_INT, H_SB, 
                                            NODE_VALUE_INT_dup(entity),0));
           if (mask & 0x00000040) {
@@ -1476,7 +1340,7 @@ node* DM2_bin_to_node(BB_t *m, int opt)
           if (mask & 0x08000000) 
             tn = node_link(tn,node_command_init(TOKEN_SOLID, V_INT, H_SHORT, 
                                              NODE_VALUE_INT_dup(solid),0));
-
+          
           ttn=node_link(ttn,node_init_all(TOKEN_PACKETENTITY, 0, tn, 0));
         }
         n=node_link(n,node_init_all(TOKEN_PACKETENTITIES, H_DM2_PACKETENTITIES, ttn, 0));
@@ -1638,13 +1502,13 @@ node* DM2_bin_to_node(BB_t *m, int opt)
 
 node* DM2_block_write_text(node* b)
 {
-  /* b may be NULL (something was wrong) */
-  if (b != NULL) {
-    /* type is TOKEN_ENDBLOCK or TOKEN_BLOCK, down is first message */
-    if ( b->type == TOKEN_BETWEENBLOCK || b->type == TOKEN_ENDBLOCK ||
-        (b->type == TOKEN_BLOCK && b->down!=NULL)) {
-      node_write_text(b,0);
-    }
+  /* b may be NULL (something was wrong or removed) */
+  if (b == NULL) return b;
+  /* type is TOKEN_BETWEENBLOCK, TOKEN_ENDBLOCK or 
+     TOKEN_BLOCK with down as first message */
+  if ( b->type == TOKEN_BETWEENBLOCK || b->type == TOKEN_ENDBLOCK ||
+      (b->type == TOKEN_BLOCK && b->down!=NULL)) {
+    node_write_text(b,0);
   }
   return b;
 }
@@ -1657,15 +1521,19 @@ node* DM2_block_write_bin(node* b)
 {
   BB_t m;
   node* tn;
-  long datasize;
+  long datasize = 0;
 
-  datasize = 0;
+  /* b may be NULL (something was wrong or removed) */
+  if (b == NULL) return b;
 
   /* we start from the beginning */
   m.p = m.buffer;
 
   /* fprintf(stderr,"block out\n"); */ 
   switch (b->type) {
+    case GEN_NOTHING:
+      /* empty blocks */
+    break;
     case TOKEN_BETWEENBLOCK:
       /* calculate the size info */
       datasize = 4;
@@ -1733,7 +1601,7 @@ node* DM2_block_write_bin(node* b)
   } 
   
   if (datasize) { /* if there was something in the block */
-    if (fwrite(m.buffer, 1, datasize, output_file) != datasize) 
+    if (fwrite(m.buffer, 1, datasize, output_file) != (size_t)datasize) 
       syserror(FIWRITE, output_filename);
     if (fflush(output_file) != 0)
       syserror(errno, output_filename);
@@ -1764,6 +1632,7 @@ void do_dm2_sound_message_write_bin(node* n, BB_t* m)
   mask = MASK_DM2_SOUND_EMPTY;
   for (cn=n->down ; cn != NULL ; NEXT(cn)) {
     switch (cn->type) {
+      case TOKEN_UNKNOWN_MASK_BITS: mask |= nodes_to_bitlist(cn->down); break;
       case TOKEN_VOL:		mask |= MASK_DM2_SOUND_VOL;		break;
       case TOKEN_ATTENUATION:	mask |= MASK_DM2_SOUND_ATTENUATION;	break;
       case TOKEN_TIMEOFS:	mask |= MASK_DM2_SOUND_TIMEOFS;		break;
@@ -1775,6 +1644,8 @@ void do_dm2_sound_message_write_bin(node* n, BB_t* m)
 
   /* output */
   WriteByte(m,mask); cn=n->down;
+  /* unknown bits for the bit mask */
+  if (cn->type == TOKEN_UNKNOWN_MASK_BITS) NEXTRET(cn);
   /* soundnum */
   do_simple_argument_write_bin(cn->down, m); NEXTRET(cn);
   if (mask & MASK_DM2_SOUND_VOL) {
@@ -1955,7 +1826,7 @@ int do_dm2_packetmask_write_bin(node* n, BB_t* m)
 {
   node* cn;
   unsigned long mask;
-  long skin = 0;
+  unsigned long skin = 0;
     
   /*
     I don't do any kind of check in here. If the internal structure is
@@ -1966,6 +1837,7 @@ int do_dm2_packetmask_write_bin(node* n, BB_t* m)
   mask = MASK_DM2_PACKET_EMPTY;
   for ( cn = n->down ; cn != NULL ; cn = cn->next ) {
     switch (cn->type) {
+      case TOKEN_UNKNOWN_MASK_BITS: mask |= nodes_to_bitlist(cn->down); break;
       case TOKEN_ENTITY:
         if (*(long*)cn->down->down > 255)
           mask |= MASK_DM2_PACKET_ENTITY_SHORT;
@@ -2068,6 +1940,12 @@ int do_dm2_packetmask_write_bin(node* n, BB_t* m)
   if (mask & MASK_DM2_PACKET_PART3) WriteByte(m, 0x000000FF & (mask >> 16));
   if (mask & MASK_DM2_PACKET_PART4) WriteByte(m, 0x000000FF & (mask >> 24));
 
+  /*
+  if (skin) {
+    printf("mask=0x%08lx, skin=%li\n",mask,skin);
+  }
+  */
+
   /* return it, so we can simpler continue with the data */
   return mask;
 }
@@ -2090,17 +1968,19 @@ void do_dm2_packetdata_write_bin(node* n, BB_t* m, unsigned long mask)
   
   cn=n->down;
 
+  /* unknown bits for the bit mask */
+  if (cn->type == TOKEN_UNKNOWN_MASK_BITS) NEXT(cn);
   /* write the entity */
   /* fprintf(stderr,"entity=%d\n", *(long*)cn->down->down); */
   aaa(1);
   /* fprintf(stderr,"enthint=%d\n",cn->down->hint); */
-  do_simple_argument_write_bin(cn->down, m); cn=cn->next;
+  do_simple_argument_write_bin(cn->down, m); NEXT(cn);
   aaa(2);
   if (mask & MASK_DM2_PACKET_REMOVE) {
-    cn=cn->next;
+    NEXT(cn);
   }
   if (mask & MASK_DM2_PACKET_MODELINDEX) {
-    do_simple_argument_write_bin(cn->down, m); cn=cn->next;
+    do_simple_argument_write_bin(cn->down, m); NEXT(cn);
   }
   aaa(3);
   if (mask & MASK_DM2_PACKET_MODELINDEX2) {
@@ -2116,7 +1996,7 @@ void do_dm2_packetdata_write_bin(node* n, BB_t* m, unsigned long mask)
   }
   aaa(6);
   if (mask & (MASK_DM2_PACKET_FRAME)) {
-    do_simple_argument_write_bin(cn->down, m); cn=cn->next;
+    do_simple_argument_write_bin(cn->down, m); NEXT(cn);
   }
   aaa(7);
   if (mask & (MASK_DM2_PACKET_SKIN)) {
@@ -2129,6 +2009,7 @@ void do_dm2_packetdata_write_bin(node* n, BB_t* m, unsigned long mask)
       NEXT(cn);
       skin |= vwep << 8;
     }
+    
     if ((mask & MASK_DM2_PACKET_SKIN_LONG) == MASK_DM2_PACKET_SKIN_LONG)
       WriteLong(m, skin);
     else {
@@ -2265,16 +2146,29 @@ void DM2_block_edit(node *n)
 
   opt = glob_opt;
 
-  /* data block manipulation */
+  /* block manipulation */
   if (opt->option & opFixMulti) {
+    /* it repairs the multi-level play-back problem */
+    
+    START_opFixMulti:
+    if (n->type == TOKEN_BETWEENBLOCK) {
+      /* it should be NULL anyway but it doesnt hurt */
+      node_delete(n->down);
+      n->down = NULL;
+      n->type = GEN_NOTHING;
+      n->hint = H_NOTHING;
+      /* go to the next option */
+      /* goto END_opFixMulti; */
+      return;
+    }
 
-    /* should repair the multi-level play-back problem but it doesn't*/
-    restart:
     for (ni=n->down;ni!=NULL;ni=ni->next) {
       /* is this message important ? */
       if (ni->type==TOKEN_STUFFTEXT) {
         /* ni->down is V_STRING, ni->down->down is the string itself */
-        if (strcmp((char*)(ni->down->down),"reconnect\n")==0) {
+        if ((strcmp ((char*)(ni->down->down),"reconnect\n"         )==0) ||
+            (strncmp((char*)(ni->down->down),"cmd configstrings",17)==0) ||
+            (strncmp((char*)(ni->down->down),"cmd baselines"    ,13)==0)) {
           /* the whole ni must disappear */
           /* unlink this node, correct the start */
           n->down = node_unlink(n->down,ni);
@@ -2282,13 +2176,46 @@ void DM2_block_edit(node *n)
           node_delete(ni);
           /* do it again */
           /* to contine from here is possible but totally confusing */
-          goto restart;
+          goto START_opFixMulti;
         } /* end if strcmp */
+        if (strncmp((char*)(ni->down->down),"precache ",9) == 0) {
+          /* the stuff after precache has to disappear */
+          free(ni->down->down);
+          ni->down->down = (node*)NODE_VALUE_STRING_dup("precache\n");
+          /* we continue right here, no need to goto restart */
+        }
       } /* end if type */
+      if (ni->type == TOKEN_SERVERDATA) {
+        long* nt;
+        /*
+           down is TOKEN_SERVERVERSION
+           down->next is TOKEN_KEY
+           down->next->next is TOKEN_ISDEMO
+           down->next->next->down V_INT
+           down->next->next->down->down is the actual value pointer 
+        */
+        if ((*(nt = (long*)(ni->down->next->next->down->down))) == 0) *nt = 1;
+      }
+      if (ni->type == TOKEN_RECONNECT) {
+        /* the whole ni must disappear */
+        /* unlink this node, correct the start */
+        n->down = node_unlink(n->down,ni);
+        /* remove the node */
+        node_delete(ni);
+        /* do it again */
+        /* to contine from here is possible but totally confusing */
+        goto START_opFixMulti;
+      }
     } /* end for ni */
   } /* end opFixMulti */
 
-  /* other manipulations may come here */
+  /*
+  nothing comes after this, so 
+  "ANSI C forbids label at end of compound statement"
+  END_opFixMulti:
+  */
+  
+  /* other optional manipulations may come here */
   
 }
 
