@@ -1,5 +1,5 @@
 /*  lmpc -- the Little Movie Processing Centre
-    Copyright (C) 1994-98 Uwe Girlich
+    Copyright (C) 1994-99 Uwe Girlich
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
     Uwe Girlich
-    Erika-von-Brockdorff-Strasse 2
+    Dantestrasse 20
     04159 Leipzig
     Deutschland / Germany
     E-mail: Uwe.Girlich@itp.uni-leipzig.de */
@@ -31,7 +31,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#if defined(HAVE_SYS_TIME_H) && defined(TM_IN_SYS_TIME)
+#include <sys/time.h>
+#if defined(TIME_WITH_SYS_TIME)
 #include <time.h>
+#endif /* defined(TIME_WITH_SYS_TIME) */
+#else /* !defined(TM_IN_SYS_TIME) || !defined(TM_IN_SYS_TIME) */
+#include <time.h>
+#endif /* !defined(TM_IN_SYS_TIME) */
 #include <unistd.h>
 #include "tools.h"
 #include "ulmp.h"
@@ -151,7 +158,7 @@ void LMP_init(LMP_t *l, char *filename, char *mode)
 
   if ((l->filename = strdup(filename))==NULL) syserror(errno,"strdup");
   if ((l->file=fopen(l->filename, mode))==NULL) syserror(errno,l->filename);
-  if (setvbuf(l->file,NULL,_IOFBF,BUFSIZ>MYBUFSIZE?BUFSIZ:MYBUFSIZE)!=0) syserror(errno,"setvbuf");
+  if (SETVBUF(l->file,NULL,_IOFBF,BUFSIZ>MYBUFSIZE?BUFSIZ:MYBUFSIZE)!=0) syserror(errno,"setvbuf");
   if (stat(l->filename,&buf)==-1) syserror(errno,l->filename);
   l->filesize=buf.st_size;
   ft=localtime(&buf.st_mtime);
@@ -213,7 +220,7 @@ void LMP_self_determine_type(LMP_t *l)
       if (l->episode>1) game&=((~HEXEN_old)&(~HEXEN_new));
       /* DOOM_old and HERETIC don't use maps>9 */
       if (l->map>9) game&=((~DOOM_old)&(~HERETIC));
-      /* 2 at 4,6 means class=mage -> not DOOM_old or HERETIC */
+      /* 2 at 4,6 means playerclass=mage -> not DOOM_old or HERETIC */
       if (l->header[4]==2 || l->header[6]==2) game&=((~DOOM_old)&(~HERETIC));
       if (game==UNKNOWN) syserror(WLMP,l->filename);
       /* there are now only 9 possibilities for the bits: 
@@ -369,7 +376,7 @@ void LMP_readheader(LMP_t *l)
     for (i=0, l->playernum=0;i<l->maxplayer;i++) {
       if (l->header[l->headersize-l->maxplayer*2+i*2]) {
         l->num[l->playernum] = i;
-        l->class[l->playernum++] = l->header[l->headersize-l->maxplayer*2+i*2+1];
+        l->playerclass[l->playernum++] = l->header[l->headersize-l->maxplayer*2+i*2+1];
       }
     }
   }
@@ -466,7 +473,7 @@ void LMP_writeheader(LMP_t *l)
     }
     for (i=0;i<l->playernum;i++) {
       l->header[l->headersize-l->maxplayer*2+l->num[i]*2]   = 1;
-      l->header[l->headersize-l->maxplayer*2+l->num[i]*2+1] = l->class[i];
+      l->header[l->headersize-l->maxplayer*2+l->num[i]*2+1] = l->playerclass[i];
     }
   }
   else {
@@ -512,7 +519,7 @@ void LS_init(LS_t *l, char *filename, char *mode)
 
   if ((l->filename = strdup(filename))==NULL) syserror(errno,"strdup");
   if ((l->file=fopen(l->filename, mode))==NULL) syserror(errno,l->filename);
-  if (setvbuf(l->file,NULL,_IOFBF,BUFSIZ>MYBUFSIZE?BUFSIZ:MYBUFSIZE)!=0) syserror(errno,"setvbuf");
+  if (SETVBUF(l->file,NULL,_IOFBF,BUFSIZ>MYBUFSIZE?BUFSIZ:MYBUFSIZE)!=0) syserror(errno,"setvbuf");
   if (stat(l->filename,&buf)==-1) syserror(errno,l->filename);
   l->filesize=buf.st_size;
 }
