@@ -1,5 +1,5 @@
 /*  lmpc -- the Little Movie Processing Centre
-    Copyright (C) 1994-98 Uwe Girlich
+    Copyright (C) 1994-99 Uwe Girlich
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
     Uwe Girlich
-    Erika-von-Brockdorff-Strasse 2
+    Dantestrasse 20
     04159 Leipzig
     Deutschland / Germany
     E-mail: Uwe.Girlich@itp.uni-leipzig.de */
@@ -98,14 +98,14 @@ typedef struct {
                  int           playernum;
                  unsigned char maxplayer;
                  int           num[MAXPLAYER_GLOBAL];
-                 int           class[MAXPLAYER_GLOBAL];
+                 int           playerclass[MAXPLAYER_GLOBAL];
                  char          *filename;
                  FILE          *file;
                  long          filesize;
                  long          datasize;
                  int           year;
-                 unsigned long tics;
-                 unsigned long stics;
+                 long          tics;
+                 long          stics;
                  long          incomplete;
                  int           ticsize;
                  double        time;
